#!/usr/bin/env python3

# Usage: ./prober-origin-table.py ../paper/figures/ss_prober_asn.csv ../paper/figures/cdf_ip_occurrences/prober_syn_across_all_exp_filtered.csv ../paper/figures/cdf_ip_occurrences/probes_from_column_add_files_filtered.txt

import getopt
import os.path
import socket
import struct
import sys

import pandas as pd

_, (asn_csv_filename, *ip_filenames) = getopt.gnu_getopt(sys.argv[1:], "")

def asn_as_int(asn):
    assert asn.startswith("AS"), asn
    return int(asn[2:])
asn = pd.read_csv(asn_csv_filename)
asn["asn"] = asn["asn"].map(asn_as_int)

ip = pd.Series()
for filename in ip_filenames:
    _, ext = os.path.splitext(filename)
    if ext == ".txt":
        with open(filename) as f:
            ip = ip.append(pd.Series(line.strip() for line in f))
    elif ext == ".csv":
        ip = ip.append(pd.read_csv(filename, sep=";")["ip_src"])
    else:
        raise ValueError(f"unrecognized input filename extension in {filename}")

data = pd.DataFrame({"ip": ip}).merge(asn, left_on="ip", right_on="host", how="left")
assert len(data) == len(ip)

data["ip_bin"] = data["ip"].map(lambda ip: struct.unpack(">L", socket.inet_aton(ip))[0])
freq_table = data.groupby(["ip_bin", "asn", "ip"])["ip_bin"].count().sort_index(ascending=False).sort_values(kind="mergesort", ascending=False)

print(freq_table[0:10])
print()
print(len(freq_table) - 10 - 3, "omitted rows")
print()
print(freq_table[-3:])
